<?php
/*
	Plugin Name: Article Import
	Plugin URI: http://webous.com/products/article-import-plugin/
	Description: Plugin to mass-import articles from zip file uploaded by the user.
	Author: Val Petruchek111
	Version: 1.19asdasd
	Author URI: http://webous.com/
*/ 

function artimport_validate_categories(&$categories, $default_categories)
	{
	static $all_categories;
	if (!$all_categories)
		{
		//we need array with all valid category IDs
		if (function_exists("get_categories"))
			{
			$all_categories = array();
			$all_categories_cache = get_categories(array('get' => 'all'));
			foreach ($all_categories_cache as $cat)
				$all_categories[] = $cat->cat_ID;
			}
		else{
			$all_categories = return_categories_list();
			}
		}

	foreach ($categories as $i=>$cat)
		{
		//removing invalid IDs
		if (!in_array($cat,$all_categories))
			unset($categories[$i]);
		}

	//if we removed all or there was none
	if (empty($categories) || !is_array($categories))
		$categories = $default_categories; 
	}

function artimport_add_pages()
	{
	//add submenu item
	add_submenu_page('post.php', 'Mass Article Upload', 'Import Articles', 0, "artimport.php", 'artimport_show');
	}

function artimport_gmt_date($date)
	{//will take yyyy-mm-dd hh:ii $date and converts it to same GMT date
	list ($date, $time)	= explode (" ",$date);
	list ($y,$m,$d) = explode("-",$date);
	list ($h,$i) = explode (":",$time);
	return gmdate("Y-m-d H:i",mktime($h,$i,0,$m,$d,$y));
	}

function artimport_future($date, $freq, $already)
	{
	//takes yyyy-mm-dd $date, desired frequency $freq, number of posts $already added
	//returs yyyy-mm-dd
	//starting from plugin version 1.14 $freq can be non-integer like /10 
	//$freq value /10 means 10 posts per day
	list ($y,$m,$d) = explode("-",$date);

	if ($freq[0]!="/")
		{//old way of dealing with dates - 1 post per $freq days
		return date("Y-m-d",mktime(0,0,0,$m,$d,$y) + $freq*$already*86400);
		}
	else{//more than 1 post per day
		$perday = substr($freq,1); //converting "/10" into "10"
		return date("Y-m-d",mktime(0,0,0,$m,$d,$y) + intval($already/$perday)*86400);
		}
	}

function artimport_extract_title($s)
	{
	//extract title from smth like "01-This is my post.txt"
	preg_match ("#\d+\-(.+)\.txt#i",$s,$matches);
	return ($matches[1]) ? $matches[1] : $s; 
	}

function artimport_clean($source, $garbage)
	{
	return str_replace ($garbage, "", $source);
	}

function artimport_parse(&$post,$title,$body,$title_from_body=false,$title_from_body_words=0)
	{
	//check for UTF8 Byte-order Mark
	if (substr($post, 0, 3)=="\xef\xbb\xbf")
		$post = substr ($post, 3);

	//parsing file contents for {} tags

	$post['post_title'] = artimport_extract_title($title);
	$title_overriden = false;

	if (preg_match("#\{title\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_title'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		$title_overriden = true;
		}

	if (preg_match("#\{excerpt\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_excerpt'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		}

	if (preg_match("#\{tags\:([^\{]+)\}#i",$body,$matches))
		{
		$post['tags_input'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		}

	if (preg_match("#\{name\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_name'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		}

	if (preg_match("#\{categories\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_category'] = explode(",",$matches[1]);
		$body = artimport_clean ($body,$matches[0]);
		}

	if (preg_match("#\{date\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_date'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		}

	if (preg_match("#\{time\:([^\{]+)\}#i",$body,$matches))
		{
		$post['post_time'] = $matches[1];
		$body = artimport_clean ($body,$matches[0]);
		}

	//all-in-one-seo support

	if (preg_match_all("#\{seo\-(title|description|keywords)\:([^\}]+)\}#i",$body,$matches))
		{
		foreach ($matches[0] as $i=>$_tmp)
			{
			$post['meta'][$matches[1][$i]] = $matches[2][$i];
			$body = artimport_clean ($body,$matches[0][$i]);
			}
		}
	//all-in-one-seo support ends
	
	$post['post_content'] = trim($body);

	//title from body starts
	if ($title_from_body && !$title_overriden)
		{//we need to extract title from body if user wants it and title was not overriden inside file
		list ($_title) = explode("\n",$post['post_content']);
		$_title = trim(strip_tags($_title));
		if ($title_from_body_words)
			{//we need to take first words from title
			$_words = explode(' ',$_title);
			$_title = implode(' ',array_slice($_words,0,$title_from_body_words));
			}
		$post['post_title'] = trim($_title);

			}
	//title from body ends
	}

function convert_ansi(&$post,$title,$body,$conv_in_utf)
	{
	if ($conv_in_utf==1)
		{
		$post['post_content']=iconv('cp1251', 'utf-8', $post['post_content']);
		$post['post_title']=iconv('cp1251', 'utf-8', $post['post_title']);
		}
	}

function timer(&$time_stop)
	{
    sleep($time_stop);
	}

function artimport_select($left,$right,$extra=null,$extra2=null,$extra3=null)
	{
	//output options for selectbox
	$result = "";
	for ($i=$left;$i<=$right;$i++)
		{
		if ($extra && $i<10)
			$i = "0$i";
		$result .= "\t\t<option value='$i$extra'>$extra2$i$extra$extra3</option>\n";
		}
	return $result;
	}

function artimport_show()
	{
	//custom function to output form and process it

	global $wp_version; //we use to enable/disable certain features
	$default_post_cat = get_settings('default_category');

	$msg = array();
	$total = 0;

	if (!empty($_POST) && !$_POST['post_category'])
		$_POST['post_category'] = array($default_post_cat);

	$folder = $_POST['folder'];
	$folder_path = "./../wp-content/$folder";
	$folder_exists = ($folder && file_exists($folder_path) && is_dir($folder_path));

	list ($start_hour, ) = explode(":",$_POST['post_time_start']);
	list ($end_hour, ) = explode(":",$_POST['post_time_end']);

	if ($end_hour<$start_hour)
		$msg[] = "Начальное время должно быть раньше чем конечное";

	if (!empty($_POST) && empty($msg))
		{
		$filename = $_FILES['userfile']['tmp_name'];
		if (!@file_exists($filename) && !$folder_path)
			$msg[] = "Нет доступа к файлу (не загружен или не правильно указана папка?)";
		else{
			$cache = array();
			if (@file_exists($filename))
				{
				$zip = @zip_open($filename);
				if (!$zip) 
					$msg[] = "Не открывается зангруженный файл (битый архив?)";
				else{
					while ($zip_entry = zip_read($zip)) 
						{
						$name = zip_entry_name($zip_entry);
						if (!zip_entry_open($zip, $zip_entry, "r")) 
							$msg[] = "Не разархивируется $name";
						else{
							$cache[$name] = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
							zip_entry_close($zip_entry);
							}
						}
					zip_close($zip);
					uksort($cache,"strnatcmp");
					}
				}

			if ($folder_exists)
				{
				//code to build $cache from folder
				$d = dir($folder_path);
				while (false !== ($entry = $d->read())) 
					{
					if (is_file("$folder_path/$entry"))
						$cache[$entry] = file_get_contents("$folder_path/$entry");
					}
				$d->close();
				uksort($cache,"strnatcmp");
				}

			if (empty($cache))
				{
				$msg[] = "Нет процессов (пустая/ошибочная папка?)";
				}
			else{
				list($usec, $sec) = explode(' ', microtime());
				srand((float) $sec + ((float) $usec * 100000));
				$q = 0;
				foreach ($cache as $name=>$cached) 
					{
					$post = array();
					$post['post_status'] = $_POST['post_status'];
					$post['tags_input'] = $_POST['post_tags'];
					$post['post_category'] = $_POST['post_category'];
					$post['post_date'] = artimport_future($_POST['start_date'],$_POST['post_frequency'],$total);
					$post['post_author'] = $_POST['post_author'];

					//creating random time
					if ($_POST['post_frequency'][0]!="/")
						{
						//just need to choose random time for 1 post per day
						if ($start_hour == $end_hour)
							$post['post_time'] = $start_hour;
						else
							$post['post_time'] = $start_hour+rand()%($end_hour-$start_hour+1);
						$post['post_time'] = sprintf("%02d",$post['post_time']).":".sprintf("%02d",rand()%60);
						}
					else{
						//post frequency is more than 1 post per day - we need to split time interval
						$start_moment = 60*$start_hour; //start time in minutes 
						$end_moment = 60*($end_hour+1);  //end time in minutes
						$post_freq = substr($_POST['post_frequency'],1); //frequency - how many posts per day
						$post_time_period = intval ( ($end_moment - $start_moment) / $post_freq); //each post every $post_time_period minutes
						$post_time = $start_moment + $post_time_period * ($total % $post_freq) + rand()%$post_time_period; //choosing random time within post interval by adding previous interval + random time in current interval
						$post['post_time'] = sprintf("%02d:%02d",intval($post_time/60),$post_time % 60);
						}
					
					artimport_parse($post,$name,$cached,$_POST['title_from_body'],$_POST['title_from_body_words']);
                 	timer($_POST['time_stop']);
					artimport_validate_categories($post['post_category'],$_POST['post_category']);
                    convert_ansi($post,$name,$cached,$_POST['conv_in_utf']);
					
					$post['post_date'] .= " ".$post['post_time'];
					$post['post_date_gmt'] = artimport_gmt_date($post['post_date']);
					unset($post['post_time']);

/* bug fix with WP stopped adding slashes when itserting posts */
if (version_compare($wp_version,"1.5.2","ge"))
	{
	//this seemes to be appicable since 1.5.2
	$post['post_content'] = addslashes($post['post_content']);
	$post['post_title'] = addslashes($post['post_title']);
	if ($post['post_excerpt'])
		$post['post_excerpt'] = addslashes($post['post_excerpt']);
	}
					if ($post['post_status']=="static")
						{
						//pages do not have categories and excerpts
						unset($post['post_category']);
						unset($post['post_excerpt']);
						}
if (version_compare($wp_version,"2.1","ge"))
	{
	if ($post['post_status']=="static")
		{
		$post['post_status'] = "publish";
		$post['post_type'] = "page";
		}
	}
/* bug fix ends */  

					$meta = $post['meta'];
					unset($post['meta']);

					$res = wp_insert_post($post);

					$q++;

					if (!$res) 
						$msg[] = "<div id=\"msgmsg$q\" class=\"error\"><p>Ошибка при добавлении поста $name</p></div>";
					else{
						//adding post meta if any
						if ($meta && is_array($meta))
							{
							foreach ($meta as $metakey => $metavalue)
								add_post_meta($res, $metakey, $metavalue);
							}
						$msg[] = "<div id=\"msgmsg$q\" class=\"updated fade\"><p>$name добавлен успешно &raquo; <a href=\"post.php?action=edit&post=$res\">Редактировать этот пост</a></p></div>";
						$total++;
						}
					}
				$msg[] = "<div id=\"msgmsg_rep\" class=\"updated fade\"><p><b>Постов добавлено: $total</b> &raquo; <a href='edit.php'>Управление постами</a></p></div>";
				}
			}
		}

?>
<div>&nbsp;</div>
<div class="wrap">
<h2>Import Articles</h2>
<table border=0 cellspacing=0 cellpadding=5 align=center bgcolor=white width=100%>
<form action="<?=$_SERVER['PHP_SELF']?>?<?=getenv("QUERY_STRING")?>" method="POST" enctype="multipart/form-data">
<?php
	if (!empty($msg))
		{
?>
	<tr>
		<td colspan=2><?=implode("\n\n",$msg)?></td>
	</tr>
<?php
		}
?>
	<tr>
		<td colspan=2><i>Настройки плагина</i></td>
	</tr>
	<tr>
		<td align=right nowrap>Автор:</td>
		<td><select name=post_author>
<?php
		global $wpdb;
		$users = $wpdb->get_results("SELECT * FROM $wpdb->users ORDER BY display_name, user_login");
		foreach ($users as $user) 
			{
			echo "\t\t<option value=\"$user->ID\">$user->display_name ($user->user_login)</option>\n";
			}
?>
			</select>
		</td>
	</tr>
	<tr>
		<td align=right>Заголовок:</td>
		<td>
			<label><input type='checkbox' name=title_from_body value='1'> <small>Использовать в качестве заголовка первую строку текста из файла</small></label>
			&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
			<small>и закончить после </small> <input type='text' name=title_from_body_words value='0' size=3> <small>слов (если "0", то первая строка полностью)</small>			
		</td>
	</tr>
	<tr>
		<td align=right>UTF-8:</td>
		<td>
			<label><input type='checkbox' name=conv_in_utf value='1'> <small>Конвертировать файлы из ANSI в UTF-8 формат</small></label>
		</td>
	</tr>
	<tr>
		<td align=right>Задержка публикаций:</td>
		<td>
			<input type='text' name=time_stop value='0' size=2> <small>сек. (если "0", то без задержки)</small>
		</td>
	</tr>
	<tr>
		<td align=right>Начальная дата:</td>
		<td><input type='text' name=start_date value='<?=date("Y-m-d")?>'> <small>(гггг-мм-дд)</small></td>
	</tr>
	<tr>
		<td align=right valign=bottom>Время публикации:</td>
		<td width=100% valign=top>
			с <select name=post_time_start><?=artimport_select(0,23,":00")?></select>
			по <select id=post_time_end_id name=post_time_end><?=artimport_select(0,23,":59")?></select>
<script>
document.getElementById('post_time_end_id').value = "23:59";
</script>
		</td>
	</tr>
	<tr>
		<td align=right>Частота постинга:</td>
		<td><select name=post_frequency>
				<?php
				for ($i=10;$i>1;$i--)
					echo "\t\t\t<option value='/$i'>$i постов каждый день</option>\n";
				?>
				<option value='1' selected='selected'>1 пост каждый день</option>
				<?=artimport_select(2,30,"","1 пост каждые "," дней")?>
			</select>
		</td>
	</tr>
<?php
/* tags introduced in 2.3 */
if (version_compare($wp_version,"2.3","ge"))
	{
?>
	<tr>
		<td align=right>Тэги по умолчанию:</td>
		<td><input type='text' name=post_tags value=''> <small>разделенные запятыми, могут быть переопределены в каждом файле</small></td>
	</tr>
<?php
	}
/* tags end */
?>
	<tr>
		<td align=right>Статус:</td>
		<td><select name=post_status>
			<option value="publish">Опубликовано</option>
<?php
/* new post status introduced in 2.3 */
if (version_compare($wp_version,"2.3","ge"))
	{
?>
			<option value="pending">На рассмотрении</option>
<?php
	}
/* new status ends */
?>
			<option value="draft">Черновик</option>
			<option value="private">Личная запись</option>
			<option value="static">Статичная страница</option>
			</select>
		</td>
	</tr>
	<tr>
		<td align=right valign=top nowrap>Категория по умолчанию:</td>
		<td>
	<fieldset>
	  <legend><a href="categories.php" title="<?php _e('Manage categories') ?>"><?php _e('Categories') ?></a></legend> 
	  <div><?php dropdown_categories($default_post_cat); ?></div>
	</fieldset>
		</td>
	</tr>
	<tr>
		<td align=right>Файл:</td>
		<td>
			<small>Папка</small>
			<input type='text' name='folder' value=''> <small>в /wp-content/</small>
		</td>
	</tr>
	<tr>
		<td colspan=2 align=center><input type=submit value="Импортировать"></td>
	</tr>
</form>
</table>
</div>
<?php
	}

add_action('admin_menu', 'artimport_add_pages');

?>